=== modified file 'setup.py'
--- setup.py	2011-11-06 00:49:02 +0000
+++ setup.py	2012-02-28 04:53:55 +0000
@@ -1,33 +1,47 @@
 #!/usr/bin/python
 """GTK+ Frontends for various Bazaar commands."""
 
-from info import *
+import os
+import sys
+
+from info import bzr_plugin_version
 
 from distutils.core import setup, Command
 from distutils.command.install_data import install_data
 from distutils.command.build import build
 from distutils.command.sdist import sdist
 try:
-    from DistUtilsExtra.command import *
+    from DistUtilsExtra.command import build_i18n
 except ImportError:
-   # Python distutils extra is not available.
-    class cmd_build_i18n(Command):
+    # Python distutils extra is not available.
+    class cmd_build_i18n(build):
+        user_options = []
+
+        def initialize_options(self):
+            self.domain = None
+            self.desktop_files = None
+
+        def finalize_options(self):
+            pass
+
         def run(self):
-            print >> sys.stderr, "For internationalization support you'll need to install https://launchpad.net/python-distutils-extra"
+            print >> sys.stderr, (
+                "For internationalization support you'll need to install "
+                "https://launchpad.net/python-distutils-extra")
 else:
     # Use build_i18n from DistUtilsExtra
     cmd_build_i18n = build_i18n.build_i18n
 
-import os
-import sys
 
 class Check(Command):
     description = "Run unit tests"
 
-    user_options = []
+    user_options = [
+        ('module=', 'm', 'The test module to run'),
+        ]
 
     def initialize_options(self):
-        pass
+        self.module = 'discover'
 
     def finalize_options(self):
         pass
@@ -38,11 +52,12 @@
     def run(self):
         from bzrlib.tests import TestLoader, TestSuite, TextTestRunner
         from bzrlib.plugin import PluginImporter
-        PluginImporter.specific_paths["bzrlib.plugins.gtk"] = os.path.dirname(__file__)
+        PluginImporter.specific_paths["bzrlib.plugins.gtk"] = os.path.dirname(
+            __file__)
         from bzrlib.plugins.gtk.tests import load_tests
         suite = TestSuite()
         loader = TestLoader()
-        load_tests(suite, None, loader)
+        load_tests(suite, self.module, loader)
         runner = TextTestRunner()
         result = runner.run(suite)
         return result.wasSuccessful()
@@ -63,7 +78,8 @@
         return 'build_credits'
 
     def run(self):
-        from bzrlib.plugin import load_plugins; load_plugins()
+        from bzrlib.plugin import load_plugins
+        load_plugins()
         from bzrlib.branch import Branch
         from bzrlib.plugins.stats.cmds import find_credits
 
@@ -126,15 +142,15 @@
     version = bzr_plugin_version[:3]
     version_string = ".".join([str(x) for x in version])
     setup(
-        name = "bzr-gtk",
-        version = version_string,
-        maintainer = "Jelmer Vernooij",
-        maintainer_email = "jelmer@samba.org",
-        description = "GTK+ Frontends for various Bazaar commands",
-        license = "GNU GPL v2 or later",
-        scripts = ['bzr-handle-patch', 'bzr-notify'],
-        url = "http://bazaar-vcs.org/BzrGtk",
-        package_dir = {
+        name="bzr-gtk",
+        version=version_string,
+        maintainer="Jelmer Vernooij",
+        maintainer_email="jelmer@samba.org",
+        description="GTK+ Frontends for various Bazaar commands",
+        license="GNU GPL v2 or later",
+        scripts=['bzr-handle-patch', 'bzr-notify'],
+        url="http://bazaar-vcs.org/BzrGtk",
+        package_dir={
             "bzrlib.plugins.gtk": ".",
             "bzrlib.plugins.gtk.viz": "viz",
             "bzrlib.plugins.gtk.annotate": "annotate",
@@ -142,7 +158,7 @@
             "bzrlib.plugins.gtk.branchview": "branchview",
             "bzrlib.plugins.gtk.preferences": "preferences",
             },
-        packages = [
+        packages=[
             "bzrlib.plugins.gtk",
             "bzrlib.plugins.gtk.viz",
             "bzrlib.plugins.gtk.annotate",
@@ -150,7 +166,7 @@
             "bzrlib.plugins.gtk.branchview",
             "bzrlib.plugins.gtk.preferences",
         ],
-        data_files=[ ('share/bzr-gtk', ['credits.pickle']),
+        data_files=[('share/bzr-gtk', ['credits.pickle']),
                     ('share/bzr-gtk/icons', ['icons/commit.png',
                                              'icons/commit16.png',
                                              'icons/diff.png',
@@ -176,7 +192,8 @@
                                             'bzr-notify.desktop']),
                     ('share/application-registry', ['bzr-gtk.applications']),
                     ('share/pixmaps', ['icons/bzr-icon-64.png']),
-                    ('share/icons/hicolor/scalable/apps', ['icons/bzr-panel.svg']),
+                    ('share/icons/hicolor/scalable/apps',
+                        ['icons/bzr-panel.svg']),
                     ('share/icons/hicolor/scalable/emblems',
                      ['icons/emblem-bzr-added.svg',
                       'icons/emblem-bzr-conflict.svg',

=== modified file 'tests/__init__.py'
--- tests/__init__.py	2012-02-03 18:59:38 +0000
+++ tests/__init__.py	2012-02-27 19:21:31 +0000
@@ -14,40 +14,62 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+__all__ = [
+    'load_tests',
+    'MockMethod',
+    'MockProperty',
+    ]
+
+import os
+
 
 def load_tests(basic_tests, module, loader):
-    testmod_names = [
-        'test_annotate_config',
-        'test_avatarsbox',
-        'test_commit',
-        'test_diff',
-        'test_history',
-        'test_graphcell',
-        'test_linegraph',
-        'test_notify',
-        'test_revisionview',
-        'test_treemodel',
-        ]
-
-    basic_tests.addTest(loader.loadTestsFromModuleNames(
-            ["%s.%s" % (__name__, tmn) for tmn in testmod_names]))
+    if module == 'discover':
+        here = os.path.abspath(os.path.dirname(__file__))
+        basic_tests.addTest(loader.discover(here))
+    else:
+        full_name = "%s.%s" % (__name__, module)
+        basic_tests.addTest(loader.loadTestsFromModuleNames([full_name]))
     return basic_tests
 
 
-class MockMethod():
+class MockMethod(object):
 
     @classmethod
-    def bind(klass, test_instance, obj, method_name):
+    def bind(klass, test_instance, obj, method_name, return_value=None):
         original_method = getattr(obj, method_name)
         test_instance.addCleanup(setattr, obj, method_name, original_method)
-        setattr(obj, method_name, klass())
+        setattr(obj, method_name, klass(return_value))
 
-    def __init__(self):
+    def __init__(self, return_value=None):
         self.called = False
+        self.call_count = 0
         self.args = None
         self.kwargs = None
+        self.return_value = return_value
 
     def __call__(self, *args, **kwargs):
         self.called = True
+        self.call_count += 1
         self.args = args
         self.kwargs = kwargs
+        return self.return_value
+
+
+class MockProperty(MockMethod):
+
+    @classmethod
+    def bind(klass, test_instance, obj, method_name, return_value=None):
+        original_method = getattr(obj, method_name)
+        test_instance.addCleanup(setattr, obj, method_name, original_method)
+        mock = klass(return_value)
+        setattr(obj, method_name, property(mock.get_value, mock.set_value))
+        return mock
+
+    def get_value(self, other):
+        self.called = True
+        return self.return_value
+
+    def set_value(self, other, value):
+        self.called = True
+        self.return_value = value

=== added file 'tests/test_ui.py'
--- tests/test_ui.py	1970-01-01 00:00:00 +0000
+++ tests/test_ui.py	2012-02-28 04:53:55 +0000
@@ -0,0 +1,375 @@
+# Copyright (C) 2012 Curtis Hovey <sinzui.is@verizon.net>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Test the ui functionality."""
+
+from gi.repository import Gtk
+
+from bzrlib import (
+    tests,
+    )
+
+from bzrlib.plugins.gtk import ui
+from bzrlib.plugins.gtk.tests import (
+    MockMethod,
+    MockProperty,
+    )
+from bzrlib.progress import ProgressTask
+
+
+class MainIterationTestCase(tests.TestCase):
+
+    def test_main_iteration(self):
+        # The main_iteration decorator iterates over the pending Gtk events
+        # after calling its function so that the UI is updated too.
+        button = Gtk.ToggleButton(label='before')
+
+        def event_listener(button):
+            button.props.label = 'after'
+
+        button.connect('clicked', event_listener)
+
+        def test_func(self):
+            button.emit('clicked')
+            return True
+
+        decorated_func = ui.main_iteration(test_func)
+        result = decorated_func(object())
+        self.assertIs(True, result)
+        self.assertIs(False, Gtk.events_pending())
+        self.assertEqual('after', button.props.label)
+
+
+class PromptDialogTestCase(tests.TestCase):
+
+    def test_init(self):
+        # The text and buttons are created.
+        dialog = ui.PromptDialog('test 123')
+        self.assertEqual('test 123', dialog.props.text)
+        self.assertEqual(Gtk.MessageType.QUESTION, dialog.props.message_type)
+        buttons = dialog.get_action_area().get_children()
+        self.assertEqual('gtk-yes', buttons[0].props.label)
+        self.assertEqual('gtk-no', buttons[1].props.label)
+
+
+class InfoDialogTestCase(tests.TestCase):
+
+    def test_init(self):
+        # The text and buttons are created.
+        dialog = ui.InfoDialog('test 123')
+        self.assertEqual('test 123', dialog.props.text)
+        self.assertEqual(Gtk.MessageType.INFO, dialog.props.message_type)
+        buttons = dialog.get_action_area().get_children()
+        self.assertEqual('gtk-close', buttons[0].props.label)
+
+
+class WarningDialogTestCase(tests.TestCase):
+
+    def test_init(self):
+        # The text and buttons are created.
+        dialog = ui.WarningDialog('test 123')
+        self.assertEqual('test 123', dialog.props.text)
+        self.assertEqual(Gtk.MessageType.WARNING, dialog.props.message_type)
+        buttons = dialog.get_action_area().get_children()
+        self.assertEqual('gtk-close', buttons[0].props.label)
+
+
+class ErrorDialogTestCase(tests.TestCase):
+
+    def test_init(self):
+        # The text and buttons are created, then shown.
+        dialog = ui.ErrorDialog('test 123')
+        self.assertEqual('test 123', dialog.props.text)
+        self.assertEqual(Gtk.MessageType.ERROR, dialog.props.message_type)
+        buttons = dialog.get_action_area().get_children()
+        self.assertEqual('gtk-close', buttons[0].props.label)
+
+
+class PasswordDialogTestCase(tests.TestCase):
+
+    def test_init(self):
+        # The label, password entry, and buttons are created, then shown.
+        MockMethod.bind(self, Gtk.Box, 'show_all')
+        dialog = ui.PasswordDialog('test password')
+        content_area = dialog.get_content_area()
+        self.assertIs(True, dialog.get_content_area().show_all.called)
+        widgets = content_area.get_children()
+        self.assertEqual('test password', widgets[0].props.label)
+        self.assertEqual(False, widgets[1].props.visibility)
+        buttons = dialog.get_action_area().get_children()
+        self.assertEqual('gtk-cancel', buttons[0].props.label)
+        self.assertEqual(
+            Gtk.ResponseType.CANCEL,
+            dialog.get_response_for_widget(buttons[0]))
+        self.assertEqual('gtk-ok', buttons[1].props.label)
+        self.assertEqual(
+            Gtk.ResponseType.OK,
+            dialog.get_response_for_widget(buttons[1]))
+
+
+class GtkProgressBarTestCase(tests.TestCase):
+
+    def test_init(self):
+        progress_bar = ui.GtkProgressBar()
+        self.assertEqual(0.0, progress_bar.props.fraction)
+        self.assertIs(None, progress_bar.total)
+        self.assertIs(None, progress_bar.current)
+
+    def test_tick(self):
+        # tick() shows the widget, does one pulse, then handles the pending
+        # events in the main loop.
+        MockMethod.bind(self, ui.GtkProgressBar, 'show')
+        MockMethod.bind(self, ui.GtkProgressBar, 'pulse')
+        progress_bar = ui.GtkProgressBar()
+        progress_bar.tick()
+        self.assertIs(True, progress_bar.show.called)
+        self.assertEqual('with_main_iteration', progress_bar.tick.__name__)
+
+    def test_update_with_data(self):
+        # update() shows the widget, sets the fraction, then handles the
+        # pending events in the main loop.
+        MockMethod.bind(self, ui.GtkProgressBar, 'show')
+        progress_bar = ui.GtkProgressBar()
+        progress_bar.update(msg='test', current_cnt=5, total_cnt=10)
+        self.assertIs(True, progress_bar.show.called)
+        self.assertEqual(0.5, progress_bar.props.fraction)
+        self.assertEqual(10, progress_bar.total)
+        self.assertEqual(5, progress_bar.current)
+        self.assertEqual('with_main_iteration', progress_bar.update.__name__)
+
+    def test_update_without_data(self):
+        progress_bar = ui.GtkProgressBar()
+        progress_bar.update(current_cnt=5, total_cnt=None)
+        self.assertEqual(0.0, progress_bar.props.fraction)
+        self.assertIs(None, progress_bar.total)
+        self.assertEqual(5, progress_bar.current)
+
+    def test_update_with_insane_data(self):
+        # The fraction must be between 0.0 and 1.0.
+        progress_bar = ui.GtkProgressBar()
+        self.assertRaises(
+            ValueError, progress_bar.update, None, 20, 2)
+
+    def test_finished(self):
+        # finished() hides the widget, resets the state, then handles the
+        # pending events in the main loop.
+        MockMethod.bind(self, ui.GtkProgressBar, 'hide')
+        progress_bar = ui.GtkProgressBar()
+        progress_bar.finished()
+        self.assertIs(True, progress_bar.hide.called)
+        self.assertEqual(0.0, progress_bar.props.fraction)
+        self.assertIs(None, progress_bar.total)
+        self.assertIs(None, progress_bar.current)
+        self.assertEqual('with_main_iteration', progress_bar.finished.__name__)
+
+    def test_clear(self):
+        # clear() is synonymous with finished.
+        MockMethod.bind(self, ui.GtkProgressBar, 'finished')
+        progress_bar = ui.GtkProgressBar()
+        progress_bar.finished()
+        self.assertIs(True, progress_bar.finished.called)
+
+
+class ProgressContainerMixin:
+
+    def test_tick(self):
+        progress_widget = self.progress_container()
+        MockMethod.bind(self, progress_widget, 'show_all')
+        MockMethod.bind(self, progress_widget.pb, 'tick')
+        progress_widget.tick()
+        self.assertIs(True, progress_widget.show_all.called)
+        self.assertIs(True, progress_widget.pb.tick.called)
+
+    def test_update(self):
+        progress_widget = self.progress_container()
+        MockMethod.bind(self, progress_widget, 'show_all')
+        MockMethod.bind(self, progress_widget.pb, 'update')
+        progress_widget.update('test', 5, 10)
+        self.assertIs(True, progress_widget.show_all.called)
+        self.assertIs(True, progress_widget.pb.update.called)
+        self.assertEqual(
+            ('test', 5, 10), progress_widget.pb.update.args)
+
+    def test_finished(self):
+        progress_widget = self.progress_container()
+        MockMethod.bind(self, progress_widget, 'hide')
+        MockMethod.bind(self, progress_widget.pb, 'finished')
+        progress_widget.finished()
+        self.assertIs(True, progress_widget.hide.called)
+        self.assertIs(True, progress_widget.pb.finished.called)
+
+    def test_clear(self):
+        progress_widget = self.progress_container()
+        MockMethod.bind(self, progress_widget, 'hide')
+        MockMethod.bind(self, progress_widget.pb, 'clear')
+        progress_widget.clear()
+        self.assertIs(True, progress_widget.hide.called)
+        self.assertIs(True, progress_widget.pb.clear.called)
+
+
+class ProgressBarWindowTestCase(ProgressContainerMixin, tests.TestCase):
+
+    progress_container = ui.ProgressBarWindow
+
+    def test_init(self):
+        pb_window = ui.ProgressBarWindow()
+        self.assertEqual('Progress', pb_window.props.title)
+        self.assertEqual(
+            Gtk.WindowPosition.CENTER_ALWAYS, pb_window.props.window_position)
+        self.assertIsInstance(pb_window.pb, ui.GtkProgressBar)
+
+
+class ProgressPanelTestCase(ProgressContainerMixin, tests.TestCase):
+
+    progress_container = ui.ProgressPanel
+
+    def test_init(self):
+        pb_window = ui.ProgressPanel()
+        self.assertEqual(
+            Gtk.Orientation.HORIZONTAL, pb_window.props.orientation)
+        self.assertEqual(5, pb_window.props.spacing)
+        self.assertIsInstance(pb_window.pb, ui.GtkProgressBar)
+        widgets = pb_window.get_children()
+        # The image's stock and icon_name properties are always None?
+        self.assertIsInstance(widgets[0], Gtk.Image)
+
+
+class GtkUIFactoryTestCase(tests.TestCase):
+
+    def test__init(self):
+        ui_factory = ui.GtkUIFactory()
+        self.assertIs(None, ui_factory._progress_bar_widget)
+
+    def test_set_progress_bar_widget(self):
+        ui_factory = ui.GtkUIFactory()
+        progress_widget = ui.ProgressPanel()
+        ui_factory.set_progress_bar_widget(progress_widget)
+        self.assertIs(progress_widget, ui_factory._progress_bar_widget)
+
+    def test_get_boolean_true(self):
+        ui_factory = ui.GtkUIFactory()
+        MockMethod.bind(self, ui.PromptDialog, 'run', Gtk.ResponseType.YES)
+        boolean_value = ui_factory.get_boolean('test')
+        self.assertIs(True, ui.PromptDialog.run.called)
+        self.assertIs(True, boolean_value)
+
+    def test_get_boolean_false(self):
+        ui_factory = ui.GtkUIFactory()
+        MockMethod.bind(self, ui.PromptDialog, 'run', Gtk.ResponseType.NO)
+        boolean_value = ui_factory.get_boolean('test')
+        self.assertIs(True, ui.PromptDialog.run.called)
+        self.assertIs(False, boolean_value)
+
+    def test_show_message(self):
+        ui_factory = ui.GtkUIFactory()
+        MockMethod.bind(self, ui.InfoDialog, 'run', Gtk.ResponseType.CLOSE)
+        ui_factory.show_message('test')
+        self.assertIs(True, ui.InfoDialog.run.called)
+
+    def test_show_warning(self):
+        ui_factory = ui.GtkUIFactory()
+        MockMethod.bind(self, ui.WarningDialog, 'run', Gtk.ResponseType.CLOSE)
+        ui_factory.show_warning('test')
+        self.assertIs(True, ui.WarningDialog.run.called)
+
+    def test_show_Error(self):
+        ui_factory = ui.GtkUIFactory()
+        MockMethod.bind(self, ui.ErrorDialog, 'run', Gtk.ResponseType.CLOSE)
+        ui_factory.show_error('test')
+        self.assertIs(True, ui.ErrorDialog.run.called)
+
+    def test_show_user_warning(self):
+        ui_factory = ui.GtkUIFactory()
+        MockMethod.bind(self, ui.WarningDialog, 'run', Gtk.ResponseType.CLOSE)
+        ui_factory.show_user_warning(
+            'recommend_upgrade', current_format_name='1.0', basedir='./test')
+        self.assertIs(True, ui.WarningDialog.run.called)
+
+    def test_show_user_warning_supressed(self):
+        ui_factory = ui.GtkUIFactory()
+        ui_factory.suppressed_warnings.add('recommend_upgrade')
+        MockMethod.bind(self, ui.WarningDialog, 'run', Gtk.ResponseType.CLOSE)
+        ui_factory.show_user_warning(
+            'recommend_upgrade', current_format_name='1.0', basedir='./test')
+        self.assertIs(False, ui.WarningDialog.run.called)
+
+    def test_get_password(self):
+        ui_factory = ui.GtkUIFactory()
+        MockMethod.bind(self, ui.PasswordDialog, 'run', Gtk.ResponseType.OK)
+        mock_property = MockProperty.bind(
+            self, ui.PasswordDialog, 'passwd', 'secret')
+        password = ui_factory.get_password('test')
+        self.assertIs(True, ui.PasswordDialog.run.called)
+        self.assertIs(True, mock_property.called)
+        self.assertEqual('secret', password)
+
+    def test_progress_all_finished_with_widget(self):
+        ui_factory = ui.GtkUIFactory()
+        progress_widget = ui.ProgressPanel()
+        MockMethod.bind(self, progress_widget, 'finished')
+        ui_factory.set_progress_bar_widget(progress_widget)
+        self.assertIs(None, ui_factory._progress_all_finished())
+        self.assertIs(True, progress_widget.finished.called)
+
+    def test_progress_all_finished_without_widget(self):
+        ui_factory = ui.GtkUIFactory()
+        self.assertIs(None, ui_factory._progress_all_finished())
+
+    def test_progress_updated_with_widget(self):
+        ui_factory = ui.GtkUIFactory()
+        progress_widget = ui.ProgressPanel()
+        MockMethod.bind(self, progress_widget, 'update')
+        ui_factory.set_progress_bar_widget(progress_widget)
+        task = ProgressTask()
+        task.msg = 'test'
+        task.current_cnt = 1
+        task.total_cnt = 2
+        self.assertIs(None, ui_factory._progress_updated(task))
+        self.assertIs(True, progress_widget.update.called)
+        self.assertEqual(
+            ('test', 1, 2), progress_widget.update.args)
+
+    def test_progress_updated_without_widget(self):
+        ui_factory = ui.GtkUIFactory()
+        MockMethod.bind(self, ui.ProgressBarWindow, 'update')
+        task = ProgressTask()
+        task.msg = 'test'
+        task.current_cnt = 1
+        task.total_cnt = 2
+        self.assertIs(None, ui_factory._progress_updated(task))
+        self.assertIsInstance(
+            ui_factory._progress_bar_widget, ui.ProgressBarWindow)
+        self.assertIs(True, ui_factory._progress_bar_widget.update.called)
+        self.assertEqual(
+            ('test', 1, 2), ui_factory._progress_bar_widget.update.args)
+
+    def test_report_transport_activity_with_widget(self):
+        ui_factory = ui.GtkUIFactory()
+        progress_widget = ui.ProgressPanel()
+        MockMethod.bind(self, progress_widget, 'tick')
+        ui_factory.set_progress_bar_widget(progress_widget)
+        self.assertIs(
+            None, ui_factory.report_transport_activity(None, None, None))
+        self.assertIs(True, progress_widget.tick.called)
+
+    def test_report_transport_activity_without_widget(self):
+        ui_factory = ui.GtkUIFactory()
+        MockMethod.bind(self, ui.ProgressBarWindow, 'tick')
+        self.assertIs(
+            None, ui_factory.report_transport_activity(None, None, None))
+        self.assertIsInstance(
+            ui_factory._progress_bar_widget, ui.ProgressBarWindow)
+        self.assertIs(True, ui.ProgressBarWindow.tick.called)

=== modified file 'ui.py'
--- ui.py	2011-09-08 03:11:06 +0000
+++ ui.py	2012-02-27 21:07:38 +0000
@@ -24,18 +24,45 @@
 from bzrlib.ui import UIFactory
 
 
-class PromptDialog(Gtk.Dialog):
+def main_iteration(function):
+    def with_main_iteration(self, *args, **kwargs):
+        result = function(self, *args, **kwargs)
+        while Gtk.events_pending():
+            Gtk.main_iteration_do(False)
+        return result
+    return with_main_iteration
+
+
+class PromptDialog(Gtk.MessageDialog):
     """Prompt the user for a yes/no answer."""
 
-    def __init__(self, prompt):
-        super(PromptDialog, self).__init__()
-
-        label = Gtk.Label(label=prompt)
-        self.get_content_area().pack_start(label, True, True, 10)
-        self.get_content_area().show_all()
-
-        self.add_buttons(Gtk.STOCK_YES, Gtk.ResponseType.YES, Gtk.STOCK_NO,
-                         Gtk.ResponseType.NO)
+    def __init__(self, prompt, parent=None):
+        super(PromptDialog, self).__init__(
+            parent, Gtk.DialogFlags.DESTROY_WITH_PARENT,
+            Gtk.MessageType.QUESTION, Gtk.ButtonsType.YES_NO, prompt)
+
+
+class InfoDialog(Gtk.MessageDialog):
+    """Show the user an informational message."""
+
+    MESSAGE_TYPE = Gtk.MessageType.INFO
+
+    def __init__(self, prompt, parent=None):
+        super(InfoDialog, self).__init__(
+            parent, Gtk.DialogFlags.DESTROY_WITH_PARENT,
+            self.MESSAGE_TYPE, Gtk.ButtonsType.CLOSE, prompt)
+
+
+class WarningDialog(InfoDialog):
+    """Show the user a warning message."""
+
+    MESSAGE_TYPE = Gtk.MessageType.WARNING
+
+
+class ErrorDialog(InfoDialog):
+    """Show the user a warning message."""
+
+    MESSAGE_TYPE = Gtk.MessageType.ERROR
 
 
 class GtkProgressBar(Gtk.ProgressBar):
@@ -46,10 +73,12 @@
         self.current = None
         self.total = None
 
+    @main_iteration
     def tick(self):
         self.show()
         self.pulse()
 
+    @main_iteration
     def update(self, msg=None, current_cnt=None, total_cnt=None):
         self.show()
         if current_cnt is not None:
@@ -59,21 +88,43 @@
         if msg is not None:
             self.set_text(msg)
         if None not in (self.current, self.total):
-            self.fraction = float(self.current) / self.total
-            if self.fraction < 0.0 or self.fraction > 1.0:
-                raise AssertionError
-            self.set_fraction(self.fraction)
-        while Gtk.events_pending():
-            Gtk.main_iteration()
-
-    def finished(self):
-        self.hide()
-
-    def clear(self):
-        self.hide()
-
-
-class ProgressBarWindow(Gtk.Window):
+            fraction = float(self.current) / self.total
+            if fraction < 0.0 or fraction > 1.0:
+                raise ValueError
+            self.set_fraction(fraction)
+
+    @main_iteration
+    def finished(self):
+        self.set_fraction(0.0)
+        self.current = None
+        self.total = None
+        self.hide()
+
+    def clear(self):
+        self.finished()
+
+
+class ProgressContainerMixin:
+    """Expose GtkProgressBar methods to a container class."""
+
+    def tick(self, *args, **kwargs):
+        self.show_all()
+        self.pb.tick(*args, **kwargs)
+
+    def update(self, *args, **kwargs):
+        self.show_all()
+        self.pb.update(*args, **kwargs)
+
+    def finished(self):
+        self.hide()
+        self.pb.finished()
+
+    def clear(self):
+        self.hide()
+        self.pb.clear()
+
+
+class ProgressBarWindow(ProgressContainerMixin, Gtk.Window):
 
     def __init__(self):
         super(ProgressBarWindow, self).__init__(type=Gtk.WindowType.TOPLEVEL)
@@ -85,54 +136,20 @@
         self.resize(250, 15)
         self.set_resizable(False)
 
-    def tick(self, *args, **kwargs):
-        self.show_all()
-        self.pb.tick(*args, **kwargs)
-
-    def update(self, *args, **kwargs):
-        self.show_all()
-        self.pb.update(*args, **kwargs)
-
-    def finished(self):
-        self.pb.finished()
-        self.hide()
-        self.destroy()
-
-    def clear(self):
-        self.pb.clear()
-        self.hide()
-
-
-class ProgressPanel(Gtk.HBox):
+
+class ProgressPanel(ProgressContainerMixin, Gtk.Box):
 
     def __init__(self):
-        super(ProgressPanel, self).__init__()
+        super(ProgressPanel, self).__init__(Gtk.Orientation.HORIZONTAL, 5)
         image_loading = Gtk.Image.new_from_stock(Gtk.STOCK_REFRESH,
                                                  Gtk.IconSize.BUTTON)
         image_loading.show()
 
         self.pb = GtkProgressBar()
-        self.set_spacing(5)
         self.set_border_width(5)
         self.pack_start(image_loading, False, False, 0)
         self.pack_start(self.pb, True, True, 0)
 
-    def tick(self, *args, **kwargs):
-        self.show_all()
-        self.pb.tick(*args, **kwargs)
-
-    def update(self, *args, **kwargs):
-        self.show_all()
-        self.pb.update(*args, **kwargs)
-
-    def finished(self):
-        self.pb.finished()
-        self.hide()
-
-    def clear(self):
-        self.pb.clear()
-        self.hide()
-
 
 class PasswordDialog(Gtk.Dialog):
     """ Prompt the user for a password. """
@@ -176,6 +193,30 @@
         dialog.destroy()
         return (response == Gtk.ResponseType.YES)
 
+    def show_message(self, msg):
+        """See UIFactory.show_message."""
+        dialog = InfoDialog(msg)
+        dialog.run()
+        dialog.destroy()
+
+    def show_warning(self, msg):
+        """See UIFactory.show_warning."""
+        dialog = WarningDialog(msg)
+        dialog.run()
+        dialog.destroy()
+
+    def show_error(self, msg):
+        """See UIFactory.show_error."""
+        dialog = ErrorDialog(msg)
+        dialog.run()
+        dialog.destroy()
+
+    def show_user_warning(self, warning_id, **message_args):
+        """See UIFactory.show_user_warning."""
+        if warning_id not in self.suppressed_warnings:
+            message = self.format_user_warning(warning_id, message_args)
+            self.show_warning(message)
+
     def get_password(self, prompt='', **kwargs):
         """Prompt the user for a password.
 
@@ -183,7 +224,7 @@
         :param kwargs: Arguments which will be expanded into the prompt.
                        This lets front ends display different things if
                        they so choose.
-        :return: The password string, return None if the user 
+        :return: The password string, return None if the user
                  canceled the request.
         """
         dialog = PasswordDialog(prompt % kwargs)
@@ -196,17 +237,24 @@
             return None
 
     def _progress_all_finished(self):
-        """See UIFactory._progress_all_finished"""
+        """See UIFactory._progress_all_finished."""
         pbw = self._progress_bar_widget
         if pbw:
             pbw.finished()
 
-    def _progress_updated(self, task):
-        """See UIFactory._progress_updated"""
+    def _ensure_progress_widget(self):
         if self._progress_bar_widget is None:
-            # Default to a window since nobody gave us a better mean to report
+            # Default to a window since nobody gave us a better means to report
             # progress.
             self.set_progress_bar_widget(ProgressBarWindow())
+
+    def _progress_updated(self, task):
+        """See UIFactory._progress_updated."""
+        self._ensure_progress_widget()
         self._progress_bar_widget.update(task.msg,
                                          task.current_cnt, task.total_cnt)
 
+    def report_transport_activity(self, transport, byte_count, direction):
+        """See UIFactory.report_transport_activity."""
+        self._ensure_progress_widget()
+        self._progress_bar_widget.tick()

